# ################################################################ #
# c.make                                                           #
#                                                                  #
# Common makefile for C programs                                   #
#                                                                  #
# (c) 2002-2003 Wouter Caarls, Delft University of Technology      #
# ################################################################ #

# Variables ###################################################### #

SOURCES = $(sort $(shell ls *.c *.C *.cc *.cpp 2>/dev/null) $(EXTRA_SOURCES))
OBJECTS = $(sort $(patsubst %.c, %.o, $(patsubst %.C, %.o, $(patsubst %.cc, %.o, $(patsubst %.cpp, %.o, $(SOURCES))))) $(EXTRA_OBJECTS))
DEPFILES = $(sort $(patsubst %.c, %.d, $(patsubst %.C, %.d, $(patsubst %.cc, %.d, $(patsubst %.cpp, %.d, $(SOURCES))))))
MAKEFILE_DEPS = Makefile $(SHARE)/vars.make $(SHARE)/include.make $(SHARE)/c.make $(SHARE)/c.depend

# Compilation #################################################### #

$(filter-out $(OVERRIDE_TARGETS), all): subdirs.all $(NAME)

$(filter-out $(OVERRIDE_TARGETS), $(NAME)): $(OBJECTS)
	$(LINKER) -o $@ $(OBJECTS) $(LFLAGS)

$(filter-out $(OVERRIDE_TARGETS), %.o):: %.c $(MAKEFILE_DEPS)
	$(CC) $(CFLAGS) -o $@ -c $<
$(filter-out $(OVERRIDE_TARGETS), %.o):: %.C $(MAKEFILE_DEPS)
	$(CPP) $(CFLAGS) -o $@ -c $<
$(filter-out $(OVERRIDE_TARGETS), %.o):: %.cc $(MAKEFILE_DEPS)
	$(CPP) $(CFLAGS) -o $@ -c $<
$(filter-out $(OVERRIDE_TARGETS), %.o):: %.cpp $(MAKEFILE_DEPS)
	$(CPP) $(CFLAGS) -o $@ -c $<

# Documentation ################################################## #

$(filter-out $(OVERRIDE_TARGETS), doc): force
	cd doc ; $(DOXYGEN) doxygen.conf
	if [ -s doc/latex ]; then $(MAKE) -C doc/latex ps; fi

# Administration ################################################# #

$(filter-out $(OVERRIDE_TARGETS), clean): subdirs.clean
	rm -rf core core.[0-9]* $(NAME) *~ $(OBJECTS) $(DEPFILES) *.i $(NAME)-$(MAJOR).$(MINOR).$(REVISION).tar $(NAME)-$(MAJOR).$(MINOR).$(REVISION).tar.gz $(NAME)-$(MAJOR).$(MINOR).$(REVISION) doc/html doc/latex

$(filter-out $(OVERRIDE_TARGETS), install): subdirs.install $(NAME) $(BINARIES) $(HEADERS)
ifdef DO_PREINSTALL
	$(MAKE) -C . preinstall
endif
ifndef NO_INSTALL
	install -d $(PREFIX)/bin
	install -pc $(NAME) $(PREFIX)/bin
endif
ifdef HEADERS
	install -pc $(HEADERS) $(PREFIX)/include
endif
ifdef BINARIES
	install -pc $(BINARIES) $(PREFIX)/bin
endif
ifdef DO_POSTINSTALL
	$(MAKE) -C . postinstall
endif

$(filter-out $(OVERRIDE_TARGETS), uninstall): subdirs.uninstall
ifndef NO_INSTALL
	rm -f $(PREFIX)/bin/$(NAME)
endif
ifdef HEADERS
	rm -f $(HEADERS:%=$(PREFIX)/include/%)
endif
ifdef BINARIES
	rm -f $(BINARIES:%=$(PREFIX)/bin/%)
endif

# Distribution ################################################### #

$(filter-out $(OVERRIDE_TARGETS), subdist): clean subdirs.subdist c.subdist;

$(filter-out $(OVERRIDE_TARGETS), c.subdist): force
	mkdir -p $(DISTDIR)
	cp `find . -maxdepth 1 -name "*.[cCh]" -o -name "*.cc" -o -name "*.cpp"` $(DISTDIR)
	cp -f $(MAKEFILE_DEPS) $(DISTDIR)/$(SHAREDISTDIR)
ifdef DISTFILES
	cp --parent $(DISTFILES) $(DISTDIR)
endif
ifdef BINARIES
	cp --parent $(BINARIES) $(DISTDIR)
endif
	sed -e "s/.*SHARE *=.*/SHARE = $(SEDSHAREDISTDIR)/" Makefile > $(DISTDIR)/Makefile
	if [ -s doc ]; then cp -R doc $(DISTDIR); fi

# Dependencies ################################################### #

$(filter-out $(OVERRIDE_TARGETS), %.d):: %.c $(MAKEFILE_DEPS)
	$(SHARE)/c.depend $(CC) $(CFLAGS) $*
$(filter-out $(OVERRIDE_TARGETS), %.d):: %.C $(MAKEFILE_DEPS)
	$(SHARE)/c.depend $(CPP) $(CFLAGS) $*
$(filter-out $(OVERRIDE_TARGETS), %.d):: %.cc $(MAKEFILE_DEPS)
	$(SHARE)/c.depend $(CPP) $(CFLAGS) $*
$(filter-out $(OVERRIDE_TARGETS), %.d):: %.cpp $(MAKEFILE_DEPS)
	$(SHARE)/c.depend $(CPP) $(CFLAGS) $*

ifeq ($(filter clean uninstall dist subdist undist distclean realclean, $(MAKECMDGOALS)),)
-include $(filter %.d, $(patsubst %.c, %.d, $(patsubst %.C, %.d, $(patsubst %.cc, %.d, $(patsubst %.cpp, %.d, $(SOURCES))))))
endif
